@@ -25,7 +25,9 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
+#ifdef HAVE_STROPTS_H
 #include <stropts.h>
+#endif
 #include <string.h>
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -130,6 +132,7 @@ gdm_address_get_sockaddr_storage (GdmAddress *address)
         struct sockaddr_storage *ss;
 
         g_return_val_if_fail (address != NULL, NULL);
+        g_return_val_if_fail (address->ss != NULL, NULL);
 
         ss = g_memdup (address->ss, sizeof (struct sockaddr_storage));
 
@@ -190,6 +193,62 @@ gdm_address_equal (GdmAddress *a,
         return FALSE;
 }
 
+/* for debugging */
+static const char *
+address_family_str (GdmAddress *address)
+{
+        const char *str;
+        switch (address->ss->ss_family) {
+        case AF_INET:
+                str = "inet";
+                break;
+        case AF_INET6:
+                str = "inet6";
+                break;
+        case AF_UNIX:
+                str = "unix";
+                break;
+        case AF_UNSPEC:
+                str = "unspecified";
+                break;
+        default:
+                str = "unknown";
+                break;
+        }
+        return str;
+}
+
+static void
+_gdm_address_debug (GdmAddress *address,
+                    const char *hostname,
+                    const char *host,
+                    const char *port)
+{
+        g_debug ("Address family:%d (%s) hostname:%s host:%s port:%s local:%d loopback:%d",
+                 address->ss->ss_family,
+                 address_family_str (address) ? address_family_str (address) : "(null)",
+                 hostname ? hostname : "(null)",
+                 host ? host : "(null)",
+                 port ? port : "(null)",
+                 gdm_address_is_local (address),
+                 gdm_address_is_loopback (address));
+}
+
+void
+gdm_address_debug (GdmAddress *address)
+{
+        char *hostname;
+        char *host;
+        char *port;
+
+        gdm_address_get_hostname (address, &hostname);
+        gdm_address_get_numeric_info (address, &host, &port);
+
+        g_free (hostname);
+        g_free (host);
+        g_free (port);
+}
+
 gboolean
 gdm_address_get_hostname (GdmAddress *address,
                           char      **hostnamep)
@@ -198,13 +257,14 @@ gdm_address_get_hostname (GdmAddress *address,
         int      res;
         gboolean ret;
 
-        g_return_val_if_fail (address != NULL || address->ss != NULL, FALSE);
+        g_return_val_if_fail (address != NULL, FALSE);
+        g_return_val_if_fail (address->ss != NULL, FALSE);
 
         ret = FALSE;
 
         host [0] = '\0';
         res = getnameinfo ((const struct sockaddr *)address->ss,
-                           sizeof (struct sockaddr_storage),
+                           (int) gdm_sockaddr_len (address->ss),
                            host, sizeof (host),
                            NULL, 0,
                            0);
@@ -212,8 +272,12 @@ gdm_address_get_hostname (GdmAddress *address,
                 ret = TRUE;
                 goto done;
         } else {
-                g_warning ("Unable lookup hostname: %s", gai_strerror (res));
-                gdm_address_debug (address);
+                const char *err_msg;
+
+                err_msg = gai_strerror (res);
+                g_warning ("Unable to lookup hostname: %s",
+                        err_msg ? err_msg : "(null)");
+                _gdm_address_debug (address, NULL, NULL, NULL);
         }
 
         /* try numeric? */
@@ -236,19 +300,25 @@ gdm_address_get_numeric_info (GdmAddress *address,
         int      res;
         gboolean ret;
 
-        g_return_val_if_fail (address != NULL || address->ss != NULL, FALSE);
+        g_return_val_if_fail (address != NULL, FALSE);
+        g_return_val_if_fail (address->ss != NULL, FALSE);
 
         ret = FALSE;
 
         host [0] = '\0';
         serv [0] = '\0';
         res = getnameinfo ((const struct sockaddr *)address->ss,
-                           sizeof (struct sockaddr_storage),
+                           (int) gdm_sockaddr_len (address->ss),
                            host, sizeof (host),
                            serv, sizeof (serv),
                            NI_NUMERICHOST | NI_NUMERICSERV);
         if (res != 0) {
-                g_warning ("Unable lookup numeric info: %s", gai_strerror (res));
+                const char *err_msg;
+
+                err_msg = gai_strerror (res);
+                g_warning ("Unable to lookup numeric info: %s",
+                        err_msg ? err_msg : "(null)");
+                _gdm_address_debug (address, NULL, NULL, NULL);
         } else {
                 ret = TRUE;
         }
@@ -377,7 +447,7 @@ add_local_addrinfo (GList **list)
                          res->ai_socktype,
                          res->ai_protocol,
                          res->ai_flags,
-                         res->ai_canonname);
+                         res->ai_canonname ? res->ai_canonname : "(null)");
                 address = gdm_address_new_from_sockaddr (res->ai_addr, res->ai_addrlen);
                 *list = g_list_append (*list, address);
         }
@@ -471,57 +541,3 @@ gdm_address_free (GdmAddress *address)
         g_free (address);
 }
 
-/* for debugging */
-static const char *
-address_family_str (GdmAddress *address)
-{
-        const char *str;
-        switch (address->ss->ss_family) {
-        case AF_INET:
-                str = "inet";
-                break;
-        case AF_INET6:
-                str = "inet6";
-                break;
-        case AF_UNIX:
-                str = "unix";
-                break;
-        case AF_UNSPEC:
-                str = "unspecified";
-                break;
-        default:
-                str = "unknown";
-                break;
-        }
-        return str;
-}
-
-void
-gdm_address_debug (GdmAddress *address)
-{
-        char *hostname;
-        char *host;
-        char *port;
-
-        g_return_if_fail (address != NULL);
-
-        hostname = NULL;
-        host = NULL;
-        port = NULL;
-
-        gdm_address_get_hostname (address, &hostname);
-        gdm_address_get_numeric_info (address, &host, &port);
-
-        g_debug ("Address family:%d (%s) hostname:%s host:%s port:%s local:%d loopback:%d",
-                 address->ss->ss_family,
-                 address_family_str (address),
-                 hostname,
-                 host,
-                 port,
-                 gdm_address_is_local (address),
-                 gdm_address_is_loopback (address));
-
-        g_free (hostname);
-        g_free (host);
-        g_free (port);
-}
