@@ -31,192 +31,40 @@
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 
+#include <dbus/dbus-glib.h>
+
 #include "gdm-log.h"
 #include "gdm-common.h"
 #include "gdm-signal-handler.h"
 #include "gdm-settings-client.h"
 #include "gdm-settings-keys.h"
+#include "gdm-profile.h"
 
 #include "gdm-greeter-session.h"
 
-#define ACCESSIBILITY_KEY         "/desktop/gnome/interface/accessibility"
+#define SM_DBUS_NAME      "org.gnome.SessionManager"
+#define SM_DBUS_PATH      "/org/gnome/SessionManager"
+#define SM_DBUS_INTERFACE "org.gnome.SessionManager"
 
-static Atom AT_SPI_IOR;
+#define SM_CLIENT_DBUS_INTERFACE "org.gnome.SessionManager.ClientPrivate"
+
+static DBusGConnection *bus_connection = NULL;
+static DBusGProxy      *sm_proxy = NULL;
+static char            *client_id = NULL;
+static DBusGProxy      *client_proxy = NULL;
 
 static gboolean
-assistive_registry_launch (void)
+is_debug_set (void)
 {
-        GPid        pid;
-        GError     *error;
-        const char *command;
-        char      **argv;
-        gboolean    res;
+        gboolean debug = FALSE;
 
-        command = AT_SPI_REGISTRYD_DIR "/at-spi-registryd";
-
-        argv = NULL;
-        error = NULL;
-        res = g_shell_parse_argv (command, NULL, &argv, &error);
-        if (! res) {
-                g_warning ("Unable to parse command: %s", error->message);
-                return FALSE;
+        /* enable debugging for unstable builds */
+        if (gdm_is_version_unstable ()) {
+                return TRUE;
         }
 
-        error = NULL;
-        res = g_spawn_async (NULL,
-                             argv,
-                             NULL,
-                             G_SPAWN_SEARCH_PATH
-                             | G_SPAWN_STDOUT_TO_DEV_NULL
-                             | G_SPAWN_STDERR_TO_DEV_NULL,
-                             NULL,
-                             NULL,
-                             &pid,
-                             &error);
-        g_strfreev (argv);
-
-        if (! res) {
-                g_warning ("Unable to run command %s: %s", command, error->message);
-                return FALSE;
-        }
-
-        if (kill (pid, 0) < 0) {
-                g_warning ("at-spi-registryd not running");
-                return FALSE;
-        }
-
-        return TRUE;
-}
-
-static GdkFilterReturn
-filter_watch (GdkXEvent *xevent,
-              GdkEvent  *event,
-              gpointer   data)
-{
-        XEvent *xev = (XEvent *)xevent;
-
-        if (xev->xany.type == PropertyNotify
-            && xev->xproperty.atom == AT_SPI_IOR) {
-                gtk_main_quit ();
-
-                return GDK_FILTER_REMOVE;
-        }
-
-        return GDK_FILTER_CONTINUE;
-}
-
-static gboolean
-filter_timeout (gpointer data)
-{
-        g_warning ("The accessibility registry was not found.");
-
-        gtk_main_quit ();
-
-        return FALSE;
-}
-
-static void
-assistive_registry_start (void)
-{
-        GdkWindow *root;
-        guint      tid;
-
-        root = gdk_get_default_root_window ();
-
-        if ( ! AT_SPI_IOR) {
-                AT_SPI_IOR = XInternAtom (GDK_DISPLAY (), "AT_SPI_IOR", False);
-        }
-
-        gdk_window_set_events (root,  GDK_PROPERTY_CHANGE_MASK);
-
-        if ( ! assistive_registry_launch ()) {
-                g_warning ("The accessibility registry could not be started.");
-                return;
-        }
-
-        gdk_window_add_filter (root, filter_watch, NULL);
-        tid = g_timeout_add_seconds (5, filter_timeout, NULL);
-
-        gtk_main ();
-
-        gdk_window_remove_filter (root, filter_watch, NULL);
-        g_source_remove (tid);
-}
-
-static void
-at_set_gtk_modules (void)
-{
-        GSList     *modules_list;
-        GSList     *l;
-        const char *old;
-        char      **modules;
-        gboolean    found_gail;
-        gboolean    found_atk_bridge;
-        int         n;
-
-        n = 0;
-        modules_list = NULL;
-        found_gail = FALSE;
-        found_atk_bridge = FALSE;
-
-        if ((old = g_getenv ("GTK_MODULES")) != NULL) {
-                modules = g_strsplit (old, ":", -1);
-                for (n = 0; modules[n]; n++) {
-                        if (!strcmp (modules[n], "gail")) {
-                                found_gail = TRUE;
-                        } else if (!strcmp (modules[n], "atk-bridge")) {
-                                found_atk_bridge = TRUE;
-                        }
-
-                        modules_list = g_slist_prepend (modules_list, modules[n]);
-                        modules[n] = NULL;
-                }
-                g_free (modules);
-        }
-
-        if (!found_gail) {
-                modules_list = g_slist_prepend (modules_list, "gail");
-                ++n;
-        }
-
-        if (!found_atk_bridge) {
-                modules_list = g_slist_prepend (modules_list, "atk-bridge");
-                ++n;
-        }
-
-        modules = g_new (char *, n + 1);
-        modules[n--] = NULL;
-        for (l = modules_list; l; l = l->next) {
-                modules[n--] = g_strdup (l->data);
-        }
-
-        g_setenv ("GTK_MODULES", g_strjoinv (":", modules), TRUE);
-        g_strfreev (modules);
-        g_slist_free (modules_list);
-}
-
-static void
-load_a11y (void)
-{
-        const char        *env_a_t_support;
-        gboolean           a_t_support;
-        GConfClient       *gconf_client;
-
-        gconf_client = gconf_client_get_default ();
-
-        env_a_t_support = g_getenv ("GNOME_ACCESSIBILITY");
-        if (env_a_t_support) {
-                a_t_support = atoi (env_a_t_support);
-        } else {
-                a_t_support = gconf_client_get_bool (gconf_client, ACCESSIBILITY_KEY, NULL);
-        }
-
-        if (a_t_support) {
-                assistive_registry_start ();
-                at_set_gtk_modules ();
-        }
-
-        g_object_unref (gconf_client);
+        gdm_settings_client_get_boolean (GDM_KEY_DEBUG, &debug);
+        return debug;
 }
 
 
@@ -277,6 +125,119 @@ signal_cb (int      signo,
         return ret;
 }
 
+static gboolean
+session_manager_connect (void)
+{
+
+        if (bus_connection == NULL) {
+                GError *error;
+
+                error = NULL;
+                bus_connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+                if (bus_connection == NULL) {
+                        g_message ("Failed to connect to the session bus: %s",
+                                   error->message);
+                        g_error_free (error);
+                        exit (1);
+                }
+        }
+
+        sm_proxy = dbus_g_proxy_new_for_name (bus_connection,
+                                              SM_DBUS_NAME,
+                                              SM_DBUS_PATH,
+                                              SM_DBUS_INTERFACE);
+        return (sm_proxy != NULL);
+}
+
+static void
+stop_cb (gpointer data)
+{
+        gtk_main_quit ();
+}
+
+static gboolean
+end_session_response (gboolean is_okay, const gchar *reason)
+{
+        gboolean ret;
+        GError *error = NULL;
+
+        ret = dbus_g_proxy_call (client_proxy, "EndSessionResponse",
+                                 &error,
+                                 G_TYPE_BOOLEAN, is_okay,
+                                 G_TYPE_STRING, reason,
+                                 G_TYPE_INVALID,
+                                 G_TYPE_INVALID);
+
+        if (!ret) {
+                g_warning ("Failed to send session response %s", error->message);
+                g_error_free (error);
+        }
+
+        return ret;
+}
+
+static void
+query_end_session_cb (guint flags, gpointer data)
+{
+        end_session_response (TRUE, NULL);
+}
+
+static void
+end_session_cb (guint flags, gpointer data)
+{
+        end_session_response (TRUE, NULL);
+        gtk_main_quit ();
+}
+
+static gboolean
+register_client (void)
+{
+        GError     *error;
+        gboolean    res;
+        const char *startup_id;
+        const char *app_id;
+
+        startup_id = g_getenv ("DESKTOP_AUTOSTART_ID");
+        app_id = "gdm-simple-greeter.desktop";
+
+        error = NULL;
+        res = dbus_g_proxy_call (sm_proxy,
+                                 "RegisterClient",
+                                 &error,
+                                 G_TYPE_STRING, app_id,
+                                 G_TYPE_STRING, startup_id,
+                                 G_TYPE_INVALID,
+                                 DBUS_TYPE_G_OBJECT_PATH, &client_id,
+                                 G_TYPE_INVALID);
+        if (! res) {
+                g_warning ("Failed to register client: %s", error->message);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        g_debug ("Client registered with session manager: %s", client_id);
+        client_proxy = dbus_g_proxy_new_for_name (bus_connection,
+                                                  SM_DBUS_NAME,
+                                                  client_id,
+                                                  SM_CLIENT_DBUS_INTERFACE);
+
+        dbus_g_proxy_add_signal (client_proxy, "Stop", G_TYPE_INVALID);
+        dbus_g_proxy_connect_signal (client_proxy, "Stop",
+                                     G_CALLBACK (stop_cb), NULL, NULL);
+
+        dbus_g_proxy_add_signal (client_proxy, "QueryEndSession", G_TYPE_UINT, G_TYPE_INVALID);
+        dbus_g_proxy_connect_signal (client_proxy, "QueryEndSession",
+                                     G_CALLBACK (query_end_session_cb), NULL, NULL);
+
+        dbus_g_proxy_add_signal (client_proxy, "EndSession", G_TYPE_UINT, G_TYPE_INVALID);
+        dbus_g_proxy_connect_signal (client_proxy, "EndSession",
+                                     G_CALLBACK (end_session_cb), NULL, NULL);
+
+        g_unsetenv ("DESKTOP_AUTOSTART_ID");
+
+        return TRUE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -295,10 +256,12 @@ main (int argc, char *argv[])
 
         g_type_init ();
 
-        if (! gdm_settings_client_init (GDMCONFDIR "/gdm.schemas", "/")) {
+        gdm_profile_start ("Initializing settings client");
+        if (! gdm_settings_client_init (DATADIR "/gdm/gdm.schemas", "/")) {
                 g_critical ("Unable to initialize settings client");
                 exit (1);
         }
+        gdm_profile_end ("Initializing settings client");
 
         g_debug ("Greeter session pid=%d display=%s xauthority=%s",
                  (int)getpid (),
@@ -309,11 +272,7 @@ main (int argc, char *argv[])
         /*sleep (15);*/
 
         gdm_log_init ();
-        gdm_log_set_debug (TRUE);
-
-        gdk_init (&argc, &argv);
-
-        load_a11y ();
+        gdm_log_set_debug (is_debug_set ());
 
         gtk_init (&argc, &argv);
 
@@ -325,11 +284,13 @@ main (int argc, char *argv[])
         gdm_signal_handler_add (signal_handler, SIGHUP, signal_cb, NULL);
         gdm_signal_handler_add (signal_handler, SIGUSR1, signal_cb, NULL);
 
+        gdm_profile_start ("Creating new greeter session");
         session = gdm_greeter_session_new ();
         if (session == NULL) {
                 g_critical ("Unable to create greeter session");
                 exit (1);
         }
+        gdm_profile_end ("Creating new greeter session");
 
         error = NULL;
         res = gdm_greeter_session_start (session, &error);
@@ -339,6 +300,17 @@ main (int argc, char *argv[])
                 exit (1);
         }
 
+        res = session_manager_connect ();
+        if (! res) {
+                g_warning ("Unable to connect to session manager");
+                exit (1);
+        }
+
+        res = register_client ();
+        if (! res) {
+                g_warning ("Unable to register client with session manager");
+        }
+
         gtk_main ();
 
         if (session != NULL) {
